@@ -14,10 +14,12 @@
 #include "git2/branch.h"
 #include "git2/buffer.h"
 #include "git2/checkout.h"
+#include "git2/cherrypick.h"
 #include "git2/clone.h"
 #include "git2/commit.h"
 #include "git2/common.h"
 #include "git2/config.h"
+#include "git2/describe.h"
 #include "git2/diff.h"
 #include "git2/errors.h"
 #include "git2/filter.h"
@@ -31,6 +33,7 @@
 #include "git2/notes.h"
 #include "git2/object.h"
 #include "git2/odb.h"
+#include "git2/odb_backend.h"
 #include "git2/oid.h"
 #include "git2/pack.h"
 #include "git2/patch.h"
